#!/usr/bin/perl
#
# brief: This script builds a release build from scratch and packages it
#        into an installer. You need %QTDIR% and %VSDIR% set in your environment
#
# pre-requisites: perl, tar, bzip2, Inno Setup 5
#

use File::Path;


########################################################################
if ($ARGV[0] eq "--upload")
{
    my $VERSION = getVersion();
    my $r = system( "perl",
                     "tools/breakpad-upload-symbolstore.pl", 
                     "dist/Last.fm-$VERSION-win.symbols.tar.bz2" );
    exit $r;
}
########################################################################

my $VERSION = getVersion();

print( "\nYou didn't forget to update the changelog, did you? ([Enter] if you didn't.)\n" );
$dummy = <STDIN>;

header( "Building release and installer for Last.fm-$VERSION" );

header( "Cleaning" );
run( "perl", "tools/svn-clean.pl" ) or die $! unless $ARGV[0] eq "--no-clean";
mkpath( "dist" );
mkpath( "build/syms" );

header( "Substituting strings in various files" );
my $QTDIR = quotemeta( $ENV{'QTDIR'} or 'c:\dev\qt\4.2.3-msvc.net' );
my $VSDIR = quotemeta( $ENV{'VSDIR'} or 'c:\Program Files\Microsoft Visual Studio .NET 2003' );
$QTDIR =~ s/\\/\\\\/g; #double escape \s for the shell :(
$VSDIR =~ s/\\/\\\\/g;
my $VERSION = getVersion();
@versionFiles = qw( src/container.rc
					src/trustInfo.manifest
					src/LastFMHelper/LastFMHelper.rc
					src/LastFMHelper/trustInfo.manifest
					src/CrashReporter/CrashReporter.rc
					src/CrashReporter/trustInfo.manifest
					src/Cleaner/Cleaner.rc
					src/Cleaner/trustInfo.manifest
					src/Killer/Killer.rc
					src/Killer/trustInfo.manifest
					src/Updater/Updater.rc
					src/Updater/trustInfo.manifest
					build-installer-win.iss
					build-installer-win-jp.iss );
updateVersion( @versionFiles );
if ($ARGV[0] eq "--jp")
{
    system( 'perl -pi".bak" -e "s/%QTDIR%/' . $QTDIR . '/g" build-installer-win-jp.iss' );
    system( 'perl -pi".bak" -e "s/%VSDIR%/' . $VSDIR . '/g" build-installer-win-jp.iss' );
}
else
{
    system( 'perl -pi".bak" -e "s/%QTDIR%/' . $QTDIR . '/g" build-installer-win.iss' );
    system( 'perl -pi".bak" -e "s/%VSDIR%/' . $VSDIR . '/g" build-installer-win.iss' );
}

header( "qmake" );
run( "qmake", '"CONFIG += breakpad release"', '"CONFIG -= debug debug_and_release"' ) or die $!;

header( "nmake" );
run( "nmake" ) or die $!;

header( "Translations" );
run( "perl", "tools\\tsrelease.pl" ) or die $!;

header( "Installer" );
if ($ARGV[0] eq "--jp")
{
    run( "c:/Program Files/Inno Setup 5/iscc.exe", "build-installer-win-jp.iss" ) or die $!;
}
else
{
    run( "c:/Program Files/Inno Setup 5/iscc.exe", "build-installer-win.iss" ) or die $!;
}

# We're doing this so that the version files will go back to their initial state
# of 0.0.0.0, thus enabling further rebuilds without having to do a clean
# checkout. A bit hacky, but pragmatic methinks.
header( "Reverting pre-processed version files" );
svnRevert( @versionFiles );

header( "Building symbolstore" );
dumpSyms( "bin" );
dumpSyms( "$ENV{QTDIR}/lib" );
chdir( "build/syms" );
run( "tar", "cjf", "../../dist/Last.fm-$VERSION-win.symbols.tar.bz2", "." );

header( "done!" );
print "To upload the symbols, issue the following command:\n";
print "       perl build-release-win.pl --upload";


########################################################################
sub header
{
    print "==> $_[0]\n";
}

sub getVersion
{
    open DATA, 'src/version.h' or die;
    my @lines = <DATA>;
    close( DATA );

    foreach my $line (@lines)
    {
      if ( $line =~ m/LASTFM_CLIENT_VERSION "(\d+\.\d+\.\d+\.\d+)"/ )
      {
          return $1;
      }
    }

    die;
}

sub updateVersion
{
	my @list = @_;
	foreach $file ( @list )
	{
	    system( "perl -pi\".bak\" -e \"s/0\\.0\\.0\\.0/$VERSION/g\" $file" );
        print $file . "\n";
	}
}

sub svnRevert
{
	my @list = @_;
	foreach $file ( @list )
	{
	    system( "svn revert $file" );
	}
}

sub dumpSyms
{
	($d = $_[0]) =~ s/\\/\//g; #swap \ with / as \s break the following :(

    opendir( DIR, $d );
    foreach $pdb (grep( /\.pdb$/, readdir( DIR ) ))
    {
		if ( !grep( /d4\.pdb/, $pdb ) )
		{
	        system( "perl", "tools\\breakpad-make-symbolstore.pl", "-c", "tools/dump_syms.exe", "build/syms", "$d/$pdb" );
		}
    }
    closedir( DIR );
}

sub run
{
    return system( @_ ) == 0;
}
########################################################################
